/*

	Title:		I2C Osc
	Software:	WinAvr
	Target:		ATtiny85

	$Revision: 68 $
	$Author  Satoshi "Chuck" Takagi $
	$Date: 2007-05-22 04:42:28 +0900 (, 22 5 2007) $

*/
/*
	CKSEL[3:0]  : PLL clock 16MHz
*/


#include <inttypes.h>
#include <avr/io.h>
#include <avr/eeprom.h> 
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>

#include "usiTwiSlave.h"


#define TWI_slaveAddress 0x10


// 4Kbyte 8bit sine table
#include "table.h"

struct _Phase {
	uint32_t	step;
	uint32_t	acc;
	uint16_t	freq;
};

volatile static struct _Phase Osc = { 54975581, 0, 0 };

#define NTABLE 4096
#define FS 78.125e3
const double kstep = 1024. * 1024. * (double)NTABLE / FS;



/* Timer0 for 78.125kHz sampling output   */
//   FCPU = 20MHz, 
ISR(TIM0_OVF_vect)
{
	static uint16_t ptr;
	uint16_t tmp;

	OCR1A = pgm_read_byte(&tblSine[ptr]);
//	ptr = Osc.acc>>20;
	tmp = Osc.acc / 65536;
	ptr = tmp>>4;
	Osc.acc += Osc.step;	// phase accumerator update of Modulator
}


enum { S_START = 0, S_s1 = 11, S_s2, S_s3, S_s4, S_S1 = 21, S_S2, S_r, S_R };


void init(void)
{
	// ports
	DDRB |= (1<<PB1);	// OC1A port as output

	// timers
	PLLCSR = (1<<PLLE)|(1<<PCKE);

	TCCR1 = (1<<PWM1A)|(1<<COM1A1)|(1<<CS10);
	GTCCR = 0;

	// start timer0
 	TCCR0A = 0;
	TCCR0B = (1<<CS00);

	TIMSK |= (1<<TOIE0);

	// Own TWI slave address
	usiTwiSlaveInit( TWI_slaveAddress );

}

int main(void)
{
	uint8_t temp, *p;
	static uint8_t state = S_START;
	static uint8_t rbuf[10];
	static uint8_t tbuf[10];
	uint32_t tmpstep;

	Osc.freq = (uint16_t)((double)Osc.step / kstep);

	init();
	sei();

	while (1) {

		sleep_mode();

		if( usiTwiDataInReceiveBuffer() ) {
			temp = usiTwiReceiveByte();

			switch (state) {
			case S_START:
				p = rbuf;
				*p++ = temp;
				switch (rbuf[0]) {
				case 'r':
					state = S_START;
					tmpstep = Osc.step;
					tbuf[4] = tmpstep % 256;
					tmpstep >>= 8;
					tbuf[3] = tmpstep % 256;
					tmpstep >>= 8;
					tbuf[2] = tmpstep % 256;
					tmpstep >>= 8;
					tbuf[1] = tmpstep % 256;
					usiTwiTransmitByte( tbuf[1] );
					usiTwiTransmitByte( tbuf[2] );
					usiTwiTransmitByte( tbuf[3] );
					usiTwiTransmitByte( tbuf[4] );
					break;
				case 'R':
					state = S_START;
					tbuf[1] = Osc.freq / 256;
					tbuf[2] = Osc.freq % 256;
					usiTwiTransmitByte( tbuf[1] );
					usiTwiTransmitByte( tbuf[2] );
					break;
				case 's':
					state = S_s1;
					break;
				case 'S':
					state = S_S1;
					break;
				}
				break;
			case S_s1:
				*p++ = temp;
				state = S_s2;
				break;
			case S_s2:
				*p++ = temp;
				state = S_s3;
				break;
			case S_s3:
				*p++ = temp;
				state = S_s4;
				break;
			case S_s4:
				*p++ = temp;
				state = S_START;
				Osc.step = rbuf[1] * 16777216 + rbuf[2] * 65536 + rbuf[3] * 256 + rbuf[4];
				Osc.freq = (uint16_t)((double)Osc.step / kstep);
				break;
			case S_S1:
				*p++ = temp;
				state = S_S2;
				break;
			case S_S2:
				*p++ = temp;
				state = S_START;
				Osc.freq = rbuf[1] * 256 + rbuf[2];
				Osc.step = (uint32_t)(kstep * (double)Osc.freq + 0.5);
				break;
			default:
				usiTwiTransmitByte(temp);
			}
		}
	}

	return 0;
}
